import csv
import os

from judgeval import JudgmentClient
from judgeval.data import Example
from judgeval.scorers import FaithfulnessScorer

if __name__ == "__main__":

    PATH_TO_CSV = os.path.join(os.path.dirname(__file__), "cstone_data.csv")
    client = JudgmentClient()


    with open(PATH_TO_CSV, "r") as f:
        reader = csv.reader(f)
        next(reader) # Skip the header row since we know the structure
        examples = []
        for row in reader:
            docket_id, excerpts, raw_response, quote, is_class_action, note = row

            example = Example(
                input=str(docket_id),
                actual_output=raw_response,
                retrieval_context=[excerpts]
            )
            examples.append(example)

        res = client.run_evaluation(
            examples=examples,
            scorers=[FaithfulnessScorer(threshold=1.0)],
            model="o1-preview",
            eval_run_name="cstone-basic-test",
            project_name="cornerstone_demo_new_o1",
            override=True,
            use_judgment=False,
        )




import openai
import os
import asyncio
from tavily import TavilyClient

from judgeval.common.tracer import Tracer

judgment = Tracer(project_name="travel_agent_demo")

@judgment.observe(span_type="search_tool")
def search_tavily(query):
    """Fetch travel data using Tavily API."""
    API_KEY = os.getenv("TAVILY_API_KEY")
    client = TavilyClient(api_key=API_KEY)
    results = client.search(query, num_results=3)
    return results



import os
from typing import List
import csv
import json
import anthropic
from openai import OpenAI
import openai
import time
import asyncio
from langsmith import wrappers
from langsmith import Client as langsmith_client 
import pandas as pd
from judgeval import JudgmentClient
from judgeval.data import Example
from judgeval.scorers import FaithfulnessScorer
from dotenv import load_dotenv
from patronus import Client as PatronusClient

load_dotenv()

OPENAI_API_KEY = os.getenv("OPENAI_API_KEY")

def load_data():
    """Load and parse the data from CSV file"""
    with open(os.path.join(os.path.dirname(__file__), "cstone_data.csv"), "r") as f:
        reader = csv.reader(f)
        next(reader)  # Skip header row
        data = list(reader)
    
    examples = []
    for row in data:
        docket_id, excerpts, raw_response, quote, is_class_action, note, is_hallucination = row
        example = Example(
            input=str(docket_id),
            actual_output=raw_response,
            retrieval_context=[excerpts],
        )
        examples.append(example)
        
    return examples

def run_judgment_evaluation(examples: List[Example]):
    """
    Run evaluation using JudgmentClient
    
    Args:
        examples: List of Example objects
        
    Returns:
        List of boolean values indicating if the example is a false negative
    """
    client = JudgmentClient()
    scorer = FaithfulnessScorer(threshold=1.0)
    
    output = client.run_evaluation(
        model="osiris-large",
        examples=examples,
        scorers=[scorer],
        eval_run_name="cstone-basic-test-osiris-large-1", 
        project_name="cstone_faithfulness_testing",
        override=True,
    )
    
    scores = []
    for result in output:
        score = result.scorers_data[0].score
        scores.append(score)
        
    return [score < 1 for score in scores]

def run_patronus_evaluation(examples: List[Example]):
    """
    Run evaluation using PatronusClient
    
    Args:
        examples: List of Example objects
        
    Returns:
        List of boolean values indicating if the example is a false negative
    """
    patronus_client = PatronusClient(api_key=os.getenv("PATRONUS_API_KEY"))
    scores = []
    
    for example in examples:
        result = patronus_client.evaluate(
            evaluator="lynx-small",
            criteria="patronus:hallucination",
            evaluated_model_input="Does the context identify a class action lawsuit?",
            evaluated_model_output=example.actual_output,
            evaluated_model_retrieved_context=example.retrieval_context,
            tags={"scenario": "cstone"},
        )
        scores.append(result.score_raw)

    print(f"patronus scores: {scores}")
        
    return [score < 0.9 for score in scores]

def evaluate_predictions(predictions):
    """Calculate metrics comparing predictions to gold labels"""
    df = pd.read_csv(os.path.join(os.path.dirname(__file__), "cstone_data.csv"))
    gold_labels = df['is_hallucination'].tolist()

    print(f"Gold labels: {gold_labels}")
    docket_ids = df['docket_id'].tolist()
    
    # Find false negatives
    false_negatives = [docket_id for docket_id, gold, pred in zip(docket_ids, gold_labels, predictions) 
                      if gold and not pred]
    
    # Calculate confusion matrix metrics
    TP = sum(1 for g, p in zip(gold_labels, predictions) if g and p)
    FP = sum(1 for g, p in zip(gold_labels, predictions) if not g and p) 
    TN = sum(1 for g, p in zip(gold_labels, predictions) if not g and not p)
    FN = sum(1 for g, p in zip(gold_labels, predictions) if g and not p)
    
    # Calculate final metrics
    if TP + FP == 0:
        precision = 0
    else:
        precision = TP / (TP + FP)
        
    if TP + FN == 0:
        recall = 0
    else:
        recall = TP / (TP + FN)
        
    if precision + recall == 0:
        f1 = 0
    else:
        f1 = 2 * (precision * recall) / (precision + recall)
    
    return {
        'precision': precision,
        'recall': recall,
        'f1': f1,
        'false_negatives': false_negatives
    }

if __name__ == "__main__":
    # Load data
    examples = load_data()
    
    # Run evaluations
    judgment_predictions = run_judgment_evaluation(examples)
    patronus_predictions = run_patronus_evaluation(examples)

    print(f"Judgment Predictions: {judgment_predictions}")
    print(f"Patronus Predictions: {patronus_predictions}")
    
    # Calculate and print metrics for both
    judgment_metrics = evaluate_predictions(judgment_predictions)
    patronus_metrics = evaluate_predictions(patronus_predictions)
    
    print(f"\nMetrics for Judgment:")
    print(f"Precision: {judgment_metrics['precision']}")
    print(f"Recall: {judgment_metrics['recall']}")
    print(f"F1 Score: {judgment_metrics['f1']}")
    print("\nJudgment false negative docket IDs:")
    for docket_id in judgment_metrics['false_negatives']:
        print(docket_id)
        
    print(f"\nMetrics for Patronus:")
    print(f"Precision: {patronus_metrics['precision']}")
    print(f"Recall: {patronus_metrics['recall']}")
    print(f"F1 Score: {patronus_metrics['f1']}")
    print("\nPatronus false negative docket IDs:")
    for docket_id in patronus_metrics['false_negatives']:
        print(docket_id)



from judgeval.data.datasets.dataset import EvalDataset
from judgeval.data.datasets.ground_truth import GroundTruthExample
from judgeval.data.datasets.eval_dataset_client import EvalDatasetClient

__all__ = ["EvalDataset", "EvalDatasetClient", "GroundTruthExample"]



import csv
import os
import openai
import json
from openai import OpenAI
import anthropic
import time
from together import Together
import concurrent.futures

if __name__ == "__main__":

    PATH_TO_CSV = os.path.join(os.path.dirname(__file__), "data.csv")
    WRITE_CSV = os.path.join(os.path.dirname(__file__), "results.csv")

    prompts = [
        "Given the following excerpts and classification, determine if there are contradictions and explain why.",
        "Given the following excerpts and classification, determine if there are contradictions and explain why. I have a slight suspicion that there is a contradiction, but I'm not sure.",
        "Given the following excerpts and classification, determine if there are contradictions and explain why. I am pretty sure there is a contradiction.",
    ]

    for p in prompts:
        with open(WRITE_CSV, 'a', newline='') as f:
            writer = csv.writer(f)
            writer.writerow([p])
        for i in range(1):
            with open(PATH_TO_CSV, "r") as f:
                reader = csv.reader(f)
                next(reader) # Skip the header row since we know the structure
                examples = []
                for row in reader:
                    docket_id,excerpts,LLM_raw_response,LLM_quote,is_class_action,LLM_note,final_flag,RZ_note = row
                    print(f"Docket ID: {docket_id}")
                    # print(f"Excerpts: {excerpts}")
                    # print(f"Raw Response: {LLM_raw_response}")
                    # print(f"Quote: {LLM_quote}")
                    # print(f"Is Class Action: {is_class_action}")
                    # print(f"Note: {LLM_note}")
                    # print(f"Final Flag: {final_flag}")
                    # print(f"RZ Note: {RZ_note}")


                    prompt = f"""
                    {p}
                    Excerpts: {excerpts},
                    Classification: {LLM_raw_response}
                    
                    Provide your response in JSON format (just the brackets no other text) with only one field contradiction (yes or no)
                    """
                    # Query the OpenAI API  

                    # Function to call OpenAI API
                    def call_openai(model, prompt):
                        client = openai.Client()
                        response = client.chat.completions.create(
                            model=model,
                            messages=[
                                {"role": "user", "content": prompt}
                            ]
                        )
                        return response.choices[0].message.content

                    # Function to call Anthropic API
                    def call_anthropic(prompt):
                        client = anthropic.Anthropic(api_key=os.getenv("ANTHROPIC_API_KEY"))
                        response = client.messages.create(
                            model="claude-3-5-sonnet-20241022",
                            max_tokens=1024,
                            messages=[
                                {"role": "user", "content": prompt}
                            ]
                        )
                        return response.content[0].text

                    # Prepare to collect responses
                    list_of_4o_responses = []
                    list_of_sonnet_responses = []
                    list_of_o1_responses = []

                    # Use ThreadPoolExecutor to perform API calls in parallel
                    with concurrent.futures.ThreadPoolExecutor() as executor:
                        # Prepare futures for OpenAI API calls
                        futures_4o = [executor.submit(call_openai, "gpt-4o", prompt) for _ in range(2)]
                        
                        # Prepare futures for Anthropic API calls
                        futures_sonnet = [executor.submit(call_anthropic, prompt) for _ in range(2)]
                        
                        # Prepare futures for O1 API calls
                        futures_o1 = [executor.submit(call_openai, "o1-mini", prompt) for _ in range(2)]

                        # Collect OpenAI responses
                        for future in concurrent.futures.as_completed(futures_4o):
                            list_of_4o_responses.append(future.result())

                        # Collect Anthropic responses
                        for future in concurrent.futures.as_completed(futures_sonnet):
                            list_of_sonnet_responses.append(future.result())

                        # Collect O1 responses
                        for future in concurrent.futures.as_completed(futures_o1):
                            list_of_o1_responses.append(future.result())
                    
                    print(list_of_4o_responses)
                    print(list_of_o1_responses)
                    print(list_of_sonnet_responses)

                    total_responses = list_of_4o_responses + list_of_o1_responses + list_of_sonnet_responses

                    prompt_o1_preview = f"""
                    Given the following JSONs, if at least 2 of the JSONs say there is a contradiction, return "yes", otherwise return "no". 
                    {total_responses}

                    Provide your response in JSON format (just the brackets no other text) with fields contradiction (yes or no). Make sure to only have 1 field.
                    """
                    client = openai.Client()
                    response = client.chat.completions.create(
                        model="o1-preview",
                        messages=[
                            {"role": "user", "content": prompt_o1_preview}
                        ]
                    ).choices[0].message.content

                    # client = Together()

                    # response = client.chat.completions.create(
                    #     model="deepseek-ai/DeepSeek-V3",  # or "deepseek-ai/DeepSeek-R1"
                    #     messages=[{"role": "user", "content": prompt}]
                    # )

                    # response = response.choices[0].message.content
                    response = response[response.find('{'):response.rfind('}')+1]
                    print(response)


                    # Parse the JSON response
                    try:
                        analysis = json.loads(response)
                        contradiction = analysis.get('contradiction', '')
                        # explanation = analysis.get('explanation', '')
                        
                        # Append to CSV
                        with open(WRITE_CSV, 'a', newline='') as f:
                            writer = csv.writer(f)
                            writer.writerow([docket_id, contradiction])
                        print(f"Added analysis for docket {docket_id}")
                    except json.JSONDecodeError as e:
                        print(f"Error parsing JSON response: {e}")
                    except Exception as e:
                        print(f"Error writing to CSV: {e}")
                    time.sleep(3)

        
